@@ -1245,17 +1245,7 @@ Perl_my_stat(pTHX)
 	    if (IoIFP(io)) {
 	        return (PL_laststatval = PerlLIO_fstat(PerlIO_fileno(IoIFP(io)), &PL_statcache));
             } else if (IoDIRP(io)) {
-#ifdef HAS_DIRFD
-                return (PL_laststatval = PerlLIO_fstat(dirfd(IoDIRP(io)), &PL_statcache));
-#else
-                Perl_die(aTHX_ PL_no_func, "dirfd");
-		/* NOT REACHED */
-		return 0;
-		/* Can't use NORETURN_FUNCTION_END because Perl_die is not
-		 *     __attribute__noreturn__
-		 * Can't use DIE because that does not return an integer
-		 */
-#endif
+                return (PL_laststatval = PerlLIO_fstat(my_dirfd(IoDIRP(io)), &PL_statcache));
             } else {
                 if (ckWARN2(WARN_UNOPENED,WARN_CLOSED))
                     report_evil_fh(gv, io, PL_op->op_type);
@@ -1580,7 +1570,7 @@ Perl_apply(pTHX_ I32 type, register SV **mark, register SV **sp)
     case OP_CHMOD:
 	APPLY_TAINT_PROPER();
 	if (++mark <= sp) {
-	    val = SvIVx(*mark);
+	    val = SvIV(*mark);
 	    APPLY_TAINT_PROPER();
 	    tot = sp - mark;
 	    while (++mark <= sp) {
@@ -1674,7 +1664,7 @@ nothing in the core.
 		Perl_croak(aTHX_ "Unrecognized signal name \"%s\"",s);
 	}
 	else
-	    val = SvIVx(*mark);
+	    val = SvIV(*mark);
 	APPLY_TAINT_PROPER();
 	tot = sp - mark;
 #ifdef VMS
@@ -1687,7 +1677,7 @@ nothing in the core.
 	     * CRTL's emulation of Unix-style signals and kill()
 	     */
 	    while (++mark <= sp) {
-		I32 proc = SvIVx(*mark);
+		I32 proc = SvIV(*mark);
 		register unsigned long int __vmssts;
 		APPLY_TAINT_PROPER();
 		if (!((__vmssts = sys$delprc(&proc,0)) & 1)) {
@@ -1711,7 +1701,7 @@ nothing in the core.
 	if (val < 0) {
 	    val = -val;
 	    while (++mark <= sp) {
-		const I32 proc = SvIVx(*mark);
+		const I32 proc = SvIV(*mark);
 		APPLY_TAINT_PROPER();
 #ifdef HAS_KILLPG
 		if (PerlProc_killpg(proc,val))	/* BSD */
@@ -1723,7 +1713,7 @@ nothing in the core.
 	}
 	else {
 	    while (++mark <= sp) {
-		const I32 proc = SvIVx(*mark);
+		const I32 proc = SvIV(*mark);
 		APPLY_TAINT_PROPER();
 		if (PerlProc_kill(proc, val))
 		    tot--;
@@ -1781,16 +1771,16 @@ nothing in the core.
            else {
                 Zero(&utbuf, sizeof utbuf, char);
 #ifdef HAS_FUTIMES
-		utbuf[0].tv_sec = (long)SvIVx(accessed);  /* time accessed */
+		utbuf[0].tv_sec = (long)SvIV(accessed);  /* time accessed */
 		utbuf[0].tv_usec = 0;
-		utbuf[1].tv_sec = (long)SvIVx(modified);  /* time modified */
+		utbuf[1].tv_sec = (long)SvIV(modified);  /* time modified */
 		utbuf[1].tv_usec = 0;
 #elif defined(BIG_TIME)
-                utbuf.actime = (Time_t)SvNVx(accessed);  /* time accessed */
-                utbuf.modtime = (Time_t)SvNVx(modified); /* time modified */
+                utbuf.actime = (Time_t)SvNV(accessed);  /* time accessed */
+                utbuf.modtime = (Time_t)SvNV(modified); /* time modified */
 #else
-                utbuf.actime = (Time_t)SvIVx(accessed);  /* time accessed */
-                utbuf.modtime = (Time_t)SvIVx(modified); /* time modified */
+                utbuf.actime = (Time_t)SvIV(accessed);  /* time accessed */
+                utbuf.modtime = (Time_t)SvIV(modified); /* time modified */
 #endif
             }
 	    APPLY_TAINT_PROPER();
