@@ -16,6 +16,17 @@
  *
  * When building without MULTIPLICITY, these variables will be truly global. */
 
+
+/* For historical reasons this file follows thrdvar.h into the interpeter
+   struct, and that file currently ends with 7 bytes of variables, so putting
+   one last byte here is good for alignment.  */
+
+/* This value may be set when embedding for full cleanup  */
+/* 0=none, 1=full, 2=full with checks */
+PERLVARI(Iperl_destruct_level,	U8,	0)
+
+PERLVAR(Iperldb,	U32)
+
 /* pseudo environmental stuff */
 PERLVAR(Iorigargc,	int)
 PERLVAR(Iorigargv,	char **)
@@ -27,10 +38,11 @@ PERLVAR(Idiehook,	SV *)
 PERLVAR(Iwarnhook,	SV *)
 
 /* switches */
-PERLVAR(Iminus_c,	bool)
 PERLVAR(Ipatchlevel,	SV *)
 PERLVAR(Ilocalpatches,	const char * const *)
 PERLVARI(Isplitstr,	const char *, " ")
+
+PERLVAR(Iminus_c,	bool)
 PERLVAR(Ipreprocess,	bool)
 PERLVAR(Iminus_n,	bool)
 PERLVAR(Iminus_p,	bool)
@@ -38,6 +50,7 @@ PERLVAR(Iminus_l,	bool)
 PERLVAR(Iminus_a,	bool)
 PERLVAR(Iminus_F,	bool)
 PERLVAR(Idoswitches,	bool)
+
 PERLVAR(Iminus_E,	bool)
 
 /*
@@ -51,17 +64,15 @@ The C variable which corresponds to Perl's $^W warning variable.
 */
 
 PERLVAR(Idowarn,	U8)
-PERLVAR(Iwidesyscalls,	bool)		/* unused since 5.8.1 */
 PERLVAR(Idoextract,	bool)
 PERLVAR(Isawampersand,	bool)		/* must save all match strings */
 PERLVAR(Iunsafe,	bool)
+PERLVAR(Iexit_flags,	U8)		/* was exit() unexpected, etc. */
+PERLVAR(Isrand_called,	bool)
+/* Part of internal state, but makes the 16th 1 byte variable in a row.  */
+PERLVAR(Itainting,	bool)		/* doing taint checks */
 PERLVAR(Iinplace,	char *)
 PERLVAR(Ie_script,	SV *)
-PERLVAR(Iperldb,	U32)
-
-/* This value may be set when embedding for full cleanup  */
-/* 0=none, 1=full, 2=full with checks */
-PERLVARI(Iperl_destruct_level,	int,	0)
 
 /* magical thingies */
 PERLVAR(Ibasetime,	Time_t)		/* $^T */
@@ -71,13 +82,17 @@ PERLVAR(Iformfeed,	SV *)		/* $^L */
 PERLVARI(Imaxsysfd,	I32,	MAXSYSFD)
 					/* top fd to pass to subprocesses */
 PERLVAR(Istatusvalue,	I32)		/* $? */
-PERLVAR(Iexit_flags,	U8)		/* was exit() unexpected, etc. */
 #ifdef VMS
 PERLVAR(Istatusvalue_vms,U32)
 #else
 PERLVAR(Istatusvalue_posix,I32)
 #endif
 
+#ifdef CSH
+PERLVARI(Icshlen,	I32,	0)
+PERLVARI(Icshname,	const char *,	CSH)
+#endif
+
 /* shortcuts to various I/O objects */
 PERLVAR(Istdingv,	GV *)
 PERLVAR(Istderrgv,	GV *)
@@ -139,20 +154,19 @@ PERLVAR(Iinitav,	AV *)		/* names of INIT subroutines */
 PERLVAR(Istrtab,	HV *)		/* shared string table */
 PERLVARI(Isub_generation,U32,1)		/* incr to invalidate method cache */
 
+/* funky return mechanisms */
+PERLVAR(Iforkprocess,	int)		/* so do_open |- can return proc# */
+
 /* memory management */
 PERLVAR(Isv_count,	I32)		/* how many SV* are currently allocated */
 PERLVAR(Isv_objcount,	I32)		/* how many objects are currently allocated */
 PERLVAR(Isv_root,	SV*)		/* storage for SVs belonging to interp */
 PERLVAR(Isv_arenaroot,	SV*)		/* list of areas for garbage collection */
 
-/* funky return mechanisms */
-PERLVAR(Iforkprocess,	int)		/* so do_open |- can return proc# */
-
 /* subprocess state */
 PERLVAR(Ifdpid,		AV *)		/* keep fd-to-pid mappings for my_popen */
 
 /* internal state */
-PERLVAR(Itainting,	bool)		/* doing taint checks */
 PERLVARI(Iop_mask,	char *,	NULL)	/* masked operations for safe evals */
 
 /* current interpreter roots */
@@ -167,24 +181,25 @@ PERLVARI(Icurcopdb,	COP *,	NULL)
 PERLVARI(Icopline,	line_t,	NOLINE)
 
 /* statics moved here for shared library purposes */
+PERLVARI(Igensym,	I32,	0)	/* next symbol for getsym() to define */
 PERLVAR(Ifilemode,	int)		/* so nextargv() can preserve mode */
 PERLVAR(Ilastfd,	int)		/* what to preserve mode on */
 PERLVAR(Ioldname,	char *)		/* what to preserve mode on */
 PERLVAR(IArgv,		char **)	/* stuff to free from do_aexec, vfork safe */
 PERLVAR(ICmd,		char *)		/* stuff to free from do_aexec, vfork safe */
-PERLVARI(Igensym,	I32,	0)	/* next symbol for getsym() to define */
 PERLVAR(Ipreambleav,	AV *)
-PERLVARI(Ilaststatval,	int,	-1)
-PERLVARI(Ilaststype,	I32,	OP_STAT)
 PERLVAR(Imess_sv,	SV *)
-
-/* XXX shouldn't these be per-thread? --GSAR */
 PERLVAR(Iors_sv,	SV *)		/* output record separator $\ */
+     /* Space for one more U16 here without increasing the structure size */
+PERLVAR(Ilast_lop_op,	OPCODE)		/* last list operator */
+PERLVAR(Iin_my,		U16)		/* we're compiling a "my" (or "our") declaration */
+PERLVARI(Ilaststype,	U16,	OP_STAT)
+PERLVARI(Ilaststatval,	int,	-1)
 
 /* interpreter atexit processing */
+PERLVARI(Iexitlistlen,	I32, 0)		/* length of same */
 PERLVARI(Iexitlist,	PerlExitListEntry *, NULL)
 					/* list of exit functions */
-PERLVARI(Iexitlistlen,	I32, 0)		/* length of same */
 
 /*
 =for apidoc Amn|HV*|PL_modglobal
@@ -219,11 +234,13 @@ PERLVAR(Isys_intern,	struct interp_intern)
 #endif
 
 /* more statics moved here */
-PERLVARI(Igeneration,	int,	100)	/* from op.c */
 PERLVAR(IDBcv,		CV *)		/* from perl.c */
+PERLVARI(Igeneration,	int,	100)	/* from op.c */
 
 PERLVARI(Iin_clean_objs,bool,    FALSE)	/* from sv.c */
 PERLVARI(Iin_clean_all,	bool,    FALSE)	/* from sv.c */
+PERLVAR(Inomemok,	bool)		/* let malloc context handle nomem */
+PERLVARI(Isavebegin,     bool,	FALSE)	/* save BEGINs for compiler	*/
 
 PERLVAR(Ilinestart,	char *)		/* beg. of most recently read line */
 
@@ -231,16 +248,14 @@ PERLVAR(Iuid,		Uid_t)		/* current real user id */
 PERLVAR(Ieuid,		Uid_t)		/* current effective user id */
 PERLVAR(Igid,		Gid_t)		/* current real group id */
 PERLVAR(Iegid,		Gid_t)		/* current effective group id */
-PERLVAR(Inomemok,	bool)		/* let malloc context handle nomem */
 PERLVARI(Ian,		U32,	0)	/* malloc sequence number */
 PERLVARI(Icop_seqmax,	U32,	0)	/* statement sequence number */
 PERLVARI(Ievalseq,	U32,	0)	/* eval sequence number */
-PERLVAR(Iorigenviron,	char **)
 PERLVAR(Iorigalen,	U32)
+PERLVAR(Iorigenviron,	char **)
 #ifdef PERL_USES_PL_PIDSTATUS
 PERLVAR(Ipidstatus,	HV *)		/* pid-to-status mappings for waitpid */
 #endif
-PERLVARI(Imaxo,	int,	MAXO)		/* maximum number of ops */
 PERLVAR(Iosname,	char *)		/* operating system */
 
 PERLVAR(Isighandlerp,	Sighandler_t)
@@ -250,6 +265,8 @@ PERLVARA(Ibody_roots,	PERL_ARENA_ROOTS_SIZE, void*) /* array of body roots */
 PERLVAR(Inice_chunk,	char *)		/* a nice chunk of memory to reuse */
 PERLVAR(Inice_chunk_size,	U32)	/* how nice the chunk of memory is */
 
+PERLVARI(Imaxo,	int,	MAXO)		/* maximum number of ops */
+
 PERLVARI(Irunops,	runops_proc_t,	MEMBER_TO_FPTR(RUNOPS_DEFAULT))
 
 PERLVARA(Itokenbuf,256,	char)
@@ -273,13 +290,6 @@ PERLVAR(Isv_undef,	SV)
 PERLVAR(Isv_no,		SV)
 PERLVAR(Isv_yes,	SV)
 
-#ifdef CSH
-PERLVARI(Icshname,	const char *,	CSH)
-PERLVARI(Icshlen,	I32,	0)
-#endif
-
-PERLVAR(Ilex_state,	U32)		/* next token is determined */
-
 /* What we know when we're in LEX_KNOWNEXT state. */
 #ifdef PERL_MAD
 PERLVARA(Inexttoke,5,	NEXTTOKE)	/* value of next token, if any */
@@ -290,33 +300,31 @@ PERLVARA(Inexttype,5,	I32)		/* type of next token */
 PERLVAR(Inexttoke,	I32)
 #endif
 
-PERLVAR(Ilinestr,	SV *)
 PERLVAR(Ibufptr,	char *)
 PERLVAR(Ioldbufptr,	char *)
 PERLVAR(Ioldoldbufptr,	char *)
 PERLVAR(Ibufend,	char *)
-PERLVARI(Iexpect,int,	XSTATE)		/* how to interpret ambiguous tokens */
 
+PERLVARI(Iexpect, U8,	XSTATE)		/* how to interpret ambiguous tokens */
+PERLVAR(Ilex_state,	U8)		/* next token is determined */
+PERLVAR(Ierror_count,	U8)		/* how many errors so far, max 10 */
+PERLVARI(Icv_has_eval, bool, FALSE) /* PL_compcv includes an entereval or similar */
 PERLVAR(Imulti_end,	I32)		/* last line of multi-line string */
 
-PERLVAR(Ierror_count,	I32)		/* how many errors so far, max 10 */
-PERLVAR(Isubline,	I32)		/* line this subroutine began on */
 PERLVAR(Isubname,	SV *)		/* name of current subroutine */
 
+PERLVAR(Isubline,	I32)		/* line this subroutine began on */
 PERLVAR(Imin_intro_pending,	I32)	/* start of vars to introduce */
+
 PERLVAR(Imax_intro_pending,	I32)	/* end of vars to introduce */
 PERLVAR(Ipadix,		I32)		/* max used index in current "register" pad */
+
 PERLVAR(Ipadix_floor,	I32)		/* how low may inner block reset padix */
 PERLVAR(Ipad_reset_pending,	I32)	/* reset pad on next attempted alloc */
 
 PERLVAR(Ilast_uni,	char *)		/* position of last named-unary op */
 PERLVAR(Ilast_lop,	char *)		/* position of last list operator */
-PERLVAR(Ilast_lop_op,	OPCODE)		/* last list operator */
-PERLVAR(Iin_my,		I32)		/* we're compiling a "my" (or "our") declaration */
 PERLVAR(Iin_my_stash,	HV *)		/* declared class of this "my" declaration */
-#ifdef FCRYPT
-PERLVARI(Icryptseen,	bool,	FALSE)	/* has fast crypt() been initialized? */
-#endif
 
 PERLVAR(Ihints,		U32)		/* pragma-tic compile-time flags */
 
@@ -325,24 +333,28 @@ PERLVAR(Idebug,		VOL U32)	/* flags given to -D switch */
 PERLVARI(Iamagic_generation,	long,	0)
 
 #ifdef USE_LOCALE_COLLATE
-PERLVARI(Icollation_ix,	U32,	0)	/* Collation generation index */
 PERLVAR(Icollation_name,char *)		/* Name of current collation */
-PERLVARI(Icollation_standard, bool,	TRUE)
-					/* Assume simple collation */
 PERLVAR(Icollxfrm_base,	Size_t)		/* Basic overhead in *xfrm() */
 PERLVARI(Icollxfrm_mult,Size_t,	2)	/* Expansion factor in *xfrm() */
+PERLVARI(Icollation_ix,	U32,	0)	/* Collation generation index */
+PERLVARI(Icollation_standard, bool,	TRUE)
+					/* Assume simple collation */
 #endif /* USE_LOCALE_COLLATE */
 
+
+#ifdef PERL_UTF8_CACHE_ASSERT
+PERLVARI(Iutf8cache, I8, -1)	/* Is the utf8 caching code enabled? */
+#else
+PERLVARI(Iutf8cache, I8, 1)	/* Is the utf8 caching code enabled? */
+#endif
+
 #ifdef USE_LOCALE_NUMERIC
 
-PERLVAR(Inumeric_name,	char *)		/* Name of current numeric locale */
 PERLVARI(Inumeric_standard,	bool,	TRUE)
 					/* Assume simple numerics */
 PERLVARI(Inumeric_local,	bool,	TRUE)
 					/* Assume local numerics */
-
-PERLVAR(Inumeric_compat1,		char)
-					/* Used to be numeric_radix */
+PERLVAR(Inumeric_name,	char *)		/* Name of current numeric locale */
 #endif /* !USE_LOCALE_NUMERIC */
 
 /* utf8 character classes */
@@ -365,16 +377,20 @@ PERLVAR(Iutf8_totitle,	SV *)
 PERLVAR(Iutf8_tolower,	SV *)
 PERLVAR(Iutf8_tofold,	SV *)
 PERLVAR(Ilast_swash_hv,	HV *)
-PERLVAR(Ilast_swash_klen,	U32)
-PERLVARA(Ilast_swash_key,10,	U8)
 PERLVAR(Ilast_swash_tmps,	U8 *)
 PERLVAR(Ilast_swash_slen,	STRLEN)
+PERLVARA(Ilast_swash_key,10,	U8)
+PERLVAR(Ilast_swash_klen,	U8)	/* Only needs to store 0-10  */
+
+#ifdef FCRYPT
+PERLVARI(Icryptseen,	bool,	FALSE)	/* has fast crypt() been initialized? */
+#endif
+
+PERLVARI(Iglob_index,	int,	0)
+
 
 PERLVAR(Iparser,	yy_parser *)	/* current parser state */
 
-PERLVARI(Iglob_index,	int,	0)
-PERLVAR(Isrand_called,	bool)
-PERLVARA(Iuudmap,256,	char)
 PERLVAR(Ibitcount,	char *)
 
 PERLVAR(Ipsig_ptr, SV**)
@@ -397,11 +413,15 @@ PERLVARI(Ibeginav_save, AV*, NULL)	/* save BEGIN{}s when compiling */
 
 PERLVAR(Ibody_arenas, void*) /* pointer to list of body-arenas */
 
-     /* 5.6.0 stopped here */
-
 PERLVAR(Ipsig_pend, int *)		/* per-signal "count" of pending */
 PERLVARI(Isig_pending, int,0)           /* Number if highest signal pending */
 
+
+PERLVAR(Itaint_warn,	bool)      /* taint warns instead of dying */
+PERLVAR(Iutf8locale,	bool)		/* utf8 locale detected */
+PERLVARI(Ihash_seed_set, bool, FALSE)		/* Hash initialized? */
+PERLVARI(Irehash_seed_set, bool, FALSE)	/* 582 hash initialized? */
+
 #ifdef USE_LOCALE_NUMERIC
 
 PERLVAR(Inumeric_radix_sv,	SV *)	/* The radix separator if not '.' */
@@ -418,7 +438,6 @@ PERLVAR(Iregex_padav,   AV*)		/* All regex objects */
 PERLVAR(Ireentrant_buffer, REENTR*)	/* here we store the _r buffers */
 #endif
 
-PERLVARI(Isavebegin,     bool,	FALSE)	/* save BEGINs for compiler	*/
 
 #ifdef PERL_MAD
 PERLVARI(Imadskills,	bool, FALSE)	/* preserve all syntactic info */
@@ -439,16 +458,12 @@ PERLVARI(Iencoding,    SV*, NULL)              /* character encoding */
 
 PERLVAR(Idebug_pad,	struct perl_debug_pad)	/* always needed because of the re extension */
 
-PERLVAR(Itaint_warn, bool)      /* taint warns instead of dying */
-
 #ifdef PL_OP_SLAB_ALLOC
 PERLVAR(IOpPtr,I32 **)
 PERLVARI(IOpSpace,I32,0)
 PERLVAR(IOpSlab,I32 *)
 #endif
 
-PERLVAR(Iutf8locale,	bool)		/* utf8 locale detected */
-
 PERLVAR(Iutf8_idstart,	SV *)
 PERLVAR(Iutf8_idcont,	SV *)
 
@@ -465,10 +480,10 @@ PERLVAR(Iunicode, U32)	/* Unicode features: $ENV{PERL_UNICODE} or -C */
 
 PERLVAR(Isignals, U32)	/* Using which pre-5.8 signals */
 
-PERLVAR(Istashcache,	HV *)		/* Cache to speed up S_method_common */
-
 PERLVAR(Ireentrant_retint, int)	/* Integer return value from reentrant functions */
 
+PERLVAR(Istashcache,	HV *)		/* Cache to speed up S_method_common */
+
 /* Hooks to shared SVs and locks. */
 PERLVARI(Isharehook,	share_proc_t,	MEMBER_TO_FPTR(Perl_sv_nosharing))
 PERLVARI(Ilockhook,	share_proc_t,	MEMBER_TO_FPTR(Perl_sv_nosharing))
@@ -493,16 +508,10 @@ PERLVARI(Ippid,		IV,		0)
 
 PERLVARI(Ihash_seed, UV, 0)		/* Hash initializer */
 
-PERLVARI(Ihash_seed_set, bool, FALSE)		/* Hash initialized? */
-
 PERLVAR(IDBassertion,   SV *)
 
-PERLVARI(Icv_has_eval, I32, 0) /* PL_compcv includes an entereval or similar */
-
 PERLVARI(Irehash_seed, UV, 0)		/* 582 hash initializer */
 
-PERLVARI(Irehash_seed_set, bool, FALSE)	/* 582 hash initialized? */
-
 #ifdef DEBUG_LEAKING_SCALARS_FORK_DUMP
 /* File descriptor to talk to the child which dumps scalars.  */
 PERLVARI(Idumper_fd, int, -1)
@@ -521,19 +530,19 @@ PERLVARI(Imy_cxt_keys, const char **, NULL) /* per-module array of pointers to M
 PERLVAR(Imemory_debug_header, struct perl_memory_debug_header)
 #endif
 
-#ifdef PERL_UTF8_CACHE_ASSERT
-PERLVARI(Iutf8cache, I8, -1)	/* Is the utf8 caching code enabled? */
-#else
-PERLVARI(Iutf8cache, I8, 1)	/* Is the utf8 caching code enabled? */
-#endif
-
 #ifdef PERL_DEBUG_READONLY_OPS
 PERLVARI(Islabs, I32**, NULL)	/* Array of slabs that have been allocated */
 PERLVARI(Islab_count, U32, 0)	/* Size of the array */
 #endif
 
-/* New variables must be added to the very end, before this comment,
- * for binary compatibility (the offsets of the old members must not change).
+PERLVARI(Iisarev, HV*, NULL) /* Reverse map of @ISA dependencies */
+
+/* If you are adding a U16, see the comment above on where there are 2 bytes
+   of gap which currently will be structure padding.  */
+
+/* Within a stable branch, new variables must be added to the very end, before
+ * this comment, for binary compatibility (the offsets of the old members must
+ *  not change).
  * (Don't forget to add your variable also to perl_clone()!)
  * XSUB.h provides wrapper functions via perlapi.h that make this
  * irrelevant, but not all code may be expected to #include XSUB.h.
