@@ -166,9 +166,6 @@ Deprecated.  Use C<GIMME_V> instead.
 #define OPpASSIGN_BACKWARDS	64	/* Left & right switched. */
 #define OPpASSIGN_CV_TO_GV	128	/* Possible optimisation for constants. */
 
-/* Private for OP_[AS]ASSIGN */
-#define OPpASSIGN_STATE		32	/* Assign to a "state" variable */
-
 /* Private for OP_MATCH and OP_SUBST{,CONST} */
 #define OPpRUNTIME		64	/* Pattern coming in on the stack */
 
@@ -696,7 +693,7 @@ struct token {
  * E       tr/E/R/, /E/
  * f       folded constant op
  * F       peg op for format
- * g       bareword was forced
+ * g       op was forced to be a word
  * i       if/unless modifier
  * I       if/elsif/unless statement
  * K       retired kid op
