@@ -7,6 +7,7 @@
  *    License or the Artistic License, as specified in the README file.
  *
  */
+#include "regcharclass.h"
 
 typedef OP OP_4tree;			/* Will be redefined later. */
 
@@ -172,7 +173,7 @@ struct regnode_2 {
 
 
 #define ANYOF_BITMAP_SIZE	32	/* 256 b/(8 b/B) */
-#define ANYOF_CLASSBITMAP_SIZE	 4	/* up to 32 (8*4) named classes */
+#define ANYOF_CLASSBITMAP_SIZE	 4	/* up to 40 (8*5) named classes */
 
 /* also used by trie */
 struct regnode_charclass {
@@ -421,7 +422,7 @@ EXTCONST U8 PL_varies[] = {
     BRANCH, BACK, STAR, PLUS, CURLY, CURLYX, REF, REFF, REFFL,
     WHILEM, CURLYM, CURLYN, BRANCHJ, IFTHEN, SUSPEND, CLUMP,
     NREF, NREFF, NREFFL,
-    ANYOFU, REG_ANYU,
+    ANYOFU, REG_ANYU, LNBREAK,
     0
 };
 #endif
@@ -450,7 +451,7 @@ EXTCONST regexp_engine PL_core_reg_engine = {
         Perl_regfree_internal, 
         Perl_reg_numbered_buff_get,
         Perl_reg_named_buff_get,
-        Perl_reg_qr_pkg,
+        Perl_reg_qr_package,
 #if defined(USE_ITHREADS)        
         Perl_regdupe_internal
 #endif        
@@ -784,3 +785,4 @@ re.pm, especially to the documentation.
 #endif /* DEBUG RELATED DEFINES */
 
 
+
